@@ -10,6 +10,7 @@
 #' @param fitmethod Method to fit the A-Ci curve. Either 'default' (Duursma 2015), or 'bilinear'. See Details.
 #' @param algorithm Passed to \code{\link{nls}}, sets the algorithm for finding parameter values.
 #' @param fitTPU Logical (default FALSE). Attempt to fit TPU limitation (fitmethod set to 'bilinear' automatically if used). See Details.
+#' @param alphag When estimating TPU limitation (with \code{fitTPU}), an additional parameter (see Details).
 #' @param useRd If Rd provided in data, and useRd=TRUE (default is FALSE), uses measured Rd in fit. Otherwise it is estimatied from the fit to the A-Ci curve.
 #' @param PPFD Photosynthetic photon flux density ('PAR') (mu mol m-2 s-1)
 #' @param Tleaf Leaf temperature (degrees C)
@@ -31,7 +32,7 @@
 #' When \code{citransition} is set, it splits the data into a Vcmax-limited (where Ci < citransition), and Jmax-limited region (Ci > citransition). Both parameters are then estimated separately for each region (Rd is estimated only for the Vcmax-limited region). \bold{Note} that the actual transition point as shown in the standard plot of the fitted A-Ci curve may be quite different from that provided, since the fitting method simply decides which part of the dataset to use for which limitation, it does not constrain the actual estimated transition point directly. See the example below. If \code{fitmethod="default"}, it applies non-linear regression to both parts of the data, and when fitmethod="bilinear", it uses linear regression on the linearized photosynthesis rate. Results will differ between the two methods (slightly).
 #' 
 #' \strong{TPU limitation - }
-#' Optionally, the \code{fitaci} function estimates the triose-phosphate utilization (TPU) rate. The TPU can act as another limitation on photosynthesis, and can be recognized by a 'flattening out' of the A-Ci curve at high Ci. When \code{fitTPU=TRUE}, the fitting method used will always be 'bilinear'. The TPU is estimated by trying out whether the fit improves when the last n points of the curve are TPU-limited (where n=1,2,...). When TPU is estimated, it is possible (though rare) that no points are Jmax-limited (in which case estimated Jmax will be NA). A minimum of two points is always reserved for the estimate of Vcmax and Rd. See examples.
+#' Optionally, the \code{fitaci} function estimates the triose-phosphate utilization (TPU) rate. The TPU can act as another limitation on photosynthesis, and can be recognized by a 'flattening out' of the A-Ci curve at high Ci. When \code{fitTPU=TRUE}, the fitting method used will always be 'bilinear'. The TPU is estimated by trying out whether the fit improves when the last n points of the curve are TPU-limited (where n=1,2,...). When TPU is estimated, it is possible (though rare) that no points are Jmax-limited (in which case estimated Jmax will be NA). A minimum of two points is always reserved for the estimate of Vcmax and Rd. An additional parameter (\code{alphag}) can be set that affects the behaviour at high Ci (see Ellsworth et al. 2015 for details, and also \code{\link{Photosyn}}). See examples.
 #' 
 #' \strong{Temperature correction - }
 #' When \code{Tcorrect=TRUE} (the default), Jmax and Vcmax are re-scaled to 25C, using the temperature response parameters provided (but Rd is always at measurement temperature). When \code{Tcorrect=FALSE}, estimates of all parameters are at measurement temperature. If TPU is fit, it is never corrected for temperature. Important parameters to the fit are GammaStar and Km, both of which are calculated from leaf temperature using standard formulations. Alternatively, they can be provided as known inputs.
@@ -73,10 +74,11 @@
 #' \item{citransition}{The input citransition (NA if it was not provided as input)}
 #' \item{gmeso}{The mesophyll conductance used in the fit (NA if it was not set)}
 #' \item{fitTPU}{Was TPU fit?}
-#' \item{RMSE}{The Root-mean squared error, calculated as \code{sqrt(sum((Ameas-Amodel)^2))}.
+#' \item{RMSE}{The Root-mean squared error, calculated as \code{sqrt(sum((Ameas-Amodel)^2))}.}
 #' 
 #' }
 #' @examples
+#' \dontrun{
 #' # Fit an A-Ci curve on a dataframe that contains Ci, Photo and optionally Tleaf and PPFD. 
 #' # Here, we use the built-in example dataset 'acidata1'.
 #' f <- fitaci(acidata1)
@@ -138,7 +140,7 @@
 #' # Fit TPU limitation
 #' ftpu <- fitaci(acidata1, fitTPU=TRUE, PPFD=1800, Tcorrect=TRUE)
 #' plot(ftpu)
-#' 
+#' }
 #' @export
 #' @rdname fitaci
 #' @importFrom stats lm
@@ -151,6 +153,8 @@ fitaci <- function(data,
                    fitmethod=c("default","bilinear"),
                    algorithm="default", 
                    fitTPU=FALSE,
+                   alphag=0,
+                   
                    useRd=FALSE,
                    PPFD=NULL,
                    Tleaf=NULL,
@@ -180,11 +184,11 @@ fitaci <- function(data,
   # Check data 
   if(nrow(data) == 0)Stop("No rows in data - check observations.")
   
-  # Make sure data is a dataframe
+  # Make sure data is a proper dataframe, not some tibble or other nonsense.
   data <- as.data.frame(data)
   
-  # no more extra parameters allowed 
-  if(exists("chkDots"))chkDots(...)   # depends on R 3.3.0, hence the check
+  # Extra parameters not used at the moment; warn when provided.
+  chkDots(...)
   
   # Add PPFD and Tleaf to data, if needed (uses default values, or input values)
   data <- set_PPFD(varnames, data, PPFD, quiet)
@@ -193,7 +197,7 @@ fitaci <- function(data,
   # Set measured Rd if provided (or warn when provided but not used)
   Rd_meas <- set_Rdmeas(varnames, data, useRd, citransition, quiet)
   haveRd <- !is.na(Rd_meas)
-  if(haveRd & useRd & fitmethod == "bilinear")Warning("Measured Rd not yet used with bilinear fitmethod.")
+  #if(haveRd & useRd & fitmethod == "bilinear")Warning("Measured Rd not yet used with bilinear fitmethod.")
   
   # Extract Ci and apply pressure correction
   data$Ci_original <- data[,varnames$Ci]
@@ -219,7 +223,7 @@ fitaci <- function(data,
     } 
     if(fitmethod == "bilinear"){
       
-      f <- do_fit_method_bilinear_bestcitrans(data, haveRd, fitTPU, Rd_meas, Patm, Tcorrect, algorithm,
+      f <- do_fit_method_bilinear_bestcitrans(data, haveRd, fitTPU, alphag, Rd_meas, Patm, Tcorrect, algorithm,
                                               alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,
                                               GammaStar_v, Km_v)
     }
@@ -238,7 +242,7 @@ fitaci <- function(data,
                           alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,GammaStar_v,Km_v)
     }
     if(fitmethod == "bilinear"){
-      f <- do_fit_method_bilinear(data, haveRd, Rd_meas, Patm, citransition, NULL, Tcorrect, algorithm,
+      f <- do_fit_method_bilinear(data, haveRd, alphag, Rd_meas, Patm, citransition, NULL, Tcorrect, algorithm,
                                 alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,
                                 GammaStar_v, Km_v)
     }
@@ -263,7 +267,8 @@ fitaci <- function(data,
   
   # Organize output
   l <- list()  
-  l$df <- acirun[order(acirun$Ci),]
+  runorder <- order(acirun$Ci)
+  l$df <- acirun[runorder,]
   if(!is.null(id)){
     if(any(!id %in% names(data))){
       Warning("id ignored - not all variables are in dataset provided")
@@ -293,6 +298,7 @@ fitaci <- function(data,
   formals(Photosyn)$Jmax <- l$pars[2]
   formals(Photosyn)$Rd <- l$pars[3]
   formals(Photosyn)$TPU <- f$TPU
+  formals(Photosyn)$alphag <- alphag
   formals(Photosyn)$Tcorrect <- Tcorrect
   formals(Photosyn)$alpha <- alpha
   formals(Photosyn)$theta <- theta
@@ -323,7 +329,9 @@ fitaci <- function(data,
   l$citransition <- ifelse(is.null(citransition), NA, citransition)
   l$gmeso <- ifelse(is.null(gmeso) || gmeso < 0, NA, gmeso)
   l$fitTPU <- fitTPU
+  l$alphag <- alphag
   l$RMSE <- rmse_acifit(l)
+  l$runorder <- runorder
   
   class(l) <- "acifit"
   
@@ -430,13 +438,14 @@ set_Rdmeas <- function(varnames, data, useRd, citransition, quiet){
       if(length(Rd_meas) > 1)
         Stop("If Rd provided as measured, it must be a single unique value for an A-Ci curve.")
       
-      if(Rd_meas < 0)Rd_meas <- -Rd_meas
+      # Use positive value throughout.
+      Rd_meas <- abs(Rd_meas)
       haveRd <- TRUE
       
       if(!is.null(citransition))Stop("At the moment cannot provide citransition as well as measured Rd.")
     }
     if(varnames$Rd %in% names(data) && !useRd){
-      if(!quiet)Warning("Rd found in dataset but useRd set to FALSE. Set to TRUE to use measured Rd.")
+      if(!quiet)message("Rd found in dataset but useRd set to FALSE. Set to TRUE to use measured Rd.")
     }
   }
   return(Rd_meas)
@@ -627,7 +636,7 @@ do_fit_method2 <- function(data, haveRd, Rd_meas, Patm, citransition, Tcorrect,
 
 
 
-do_fit_method_bilinear <- function(data, haveRd, Rd_meas, Patm, citransition, citransition2=NULL,
+do_fit_method_bilinear <- function(data, haveRd, alphag, Rd_meas, Patm, citransition, citransition2=NULL,
                                    Tcorrect, algorithm,
                                    alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,
                                    GammaStar, Km){
@@ -654,16 +663,25 @@ do_fit_method_bilinear <- function(data, haveRd, Rd_meas, Patm, citransition, ci
   # Linearize
   data$vcmax_pred <- (Conc - ppar$GammaStar)/(Conc + ppar$Km)
   data$Jmax_pred <- (Conc - ppar$GammaStar)/(Conc + 2*ppar$GammaStar)
-  
+  data$TPU_part <- (Conc - ppar$GammaStar)/(Conc - (1+3*alphag)*ppar$GammaStar)
+    
   # Fit Vcmax and Rd from linearized portion
   datv <- data[data$Ci < citransition & data$Ci < citransition2,]
   if(nrow(datv) == 0){
     return(list(pars=NA, fit=NA, TPU=NA, success=FALSE))
   }
-  fitv <- lm(ALEAF ~ vcmax_pred, data=datv)
   
-  Rd_fit <- coef(fitv)[[1]]
-  Vcmax_fit <- coef(fitv)[[2]]
+  if(!haveRd){
+    fitv <- lm(ALEAF ~ vcmax_pred, data=datv)
+    Rd_fit <- coef(fitv)[[1]]
+    Vcmax_fit <- coef(fitv)[[2]]
+  } else {
+    # IF using measured Rd, add to Anet, and remove intercept from fit.
+    datv$ALEAFg <- datv$ALEAF + Rd_meas
+    fitv <- lm(ALEAFg ~ vcmax_pred-1, data=datv)
+    Rd_fit <- -Rd_meas
+    Vcmax_fit <- coef(fitv)[[1]]
+  }
   
   # Fit Jmax from linearized portion
   datj <- data[data$Ci >= citransition & data$Ci < citransition2,]
@@ -679,6 +697,7 @@ do_fit_method_bilinear <- function(data, haveRd, Rd_meas, Patm, citransition, ci
   if(nrow(datj) > 0){
     
     # Fit gross photo using fitted Rd
+    # (Rd_fit is negative)
     datj$Agross <- datj$ALEAF - Rd_fit
     
     # One point, calculate directly
@@ -696,9 +715,13 @@ do_fit_method_bilinear <- function(data, haveRd, Rd_meas, Patm, citransition, ci
     Jmax_fit <- 10^6  # not elegant but will do for now (avoids trouble elsewhere)
   }
   
+  # TPU
   if(nrow(datp) > 0){
     datp$Agross <- datp$ALEAF - Rd_fit
-    TPU <- mean(datp$Agross) / 3
+    
+    tpu_vals <- (1/3) * datp$Agross / datp$TPU_part
+    
+    TPU <- mean(tpu_vals)
   } else {
     TPU <- 1000  # same as default in Photosyn
   }
@@ -711,15 +734,23 @@ do_fit_method_bilinear <- function(data, haveRd, Rd_meas, Patm, citransition, ci
   }
   
   ses <- summary(fitv)$coefficients[,2]
-  pars <- matrix(c(Vcmax_fit, Jmax_fit, -Rd_fit,
+  if(!haveRd){
+    pars <- matrix(c(Vcmax_fit, Jmax_fit, -Rd_fit,
                    ses[2],NA,ses[1]), ncol=2)
+  } else {
+    pars <- matrix(c(Vcmax_fit, Jmax_fit, -Rd_fit,
+                     ses[1],NA,NA), ncol=2)
+  }
+  
+  
   rownames(pars) <- c("Vcmax","Jmax","Rd")
   colnames(pars) <- c("Estimate","Std. Error")
   
+  
   return(list(pars=pars, fit=fitv, TPU=TPU, success=TRUE))
 }
 
-do_fit_method_bilinear_bestcitrans <- function(data, haveRd, fitTPU, Rd_meas, Patm, Tcorrect,
+do_fit_method_bilinear_bestcitrans <- function(data, haveRd, fitTPU, alphag, Rd_meas, Patm, Tcorrect,
                                                algorithm,alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,
                                                delsJ,GammaStar, Km){
   
@@ -746,7 +777,7 @@ do_fit_method_bilinear_bestcitrans <- function(data, haveRd, fitTPU, Rd_meas, Pa
   # after the loop finishes (avoids a bug).
   for(i in 1:nrow(citransdf)){
     
-    fit <- do_fit_method_bilinear(data, haveRd, Rd_meas, Patm, citransdf$ci1[i], citransdf$ci2[i], 
+    fit <- do_fit_method_bilinear(data, haveRd, alphag, Rd_meas, Patm, citransdf$ci1[i], citransdf$ci2[i], 
                                   Tcorrect=FALSE, algorithm,
                                   alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,
                                   GammaStar, Km)
@@ -767,9 +798,14 @@ do_fit_method_bilinear_bestcitrans <- function(data, haveRd, fitTPU, Rd_meas, Pa
   # Best Ci transitions
   bestcis <- citransdf[which.min(SS),]
   
-  f <- do_fit_method_bilinear(data, haveRd, Rd_meas, Patm, bestcis$ci1, bestcis$ci2, Tcorrect, algorithm,
+  f <- do_fit_method_bilinear(data, haveRd, alphag, Rd_meas, Patm, 
+                              bestcis$ci1, bestcis$ci2, Tcorrect, algorithm,
                               alpha,theta,gmeso,EaV,EdVC,delsC,EaJ,EdVJ,delsJ,
                               GammaStar, Km)
+  
+  if(f$pars["Jmax","Estimate"] < 0){
+    Stop("Cannot invert light response curve to estimate Jmax - increase alpha or theta.")
+  }
 
   
   return(f)  
