@@ -1,25 +1,27 @@
 #' @title Coupled leaf gas exchange model
-#' @description A coupled photosynthesis - stomatal conductance model, based on the Farquhar model of photosynthesis, and a Ball-Berry type model of stomatatal conductance. Includes options for temperature sensitivity of photosynthetic parameters, dark respiration (optionally calculated from leaf temperature), and mesophyll conductance. 
+#' @description A coupled photosynthesis - stomatal conductance model, based on the Farquhar model of photosynthesis, and a Ball-Berry type model of stomatatal conductance. Includes options for temperature sensitivity of photosynthetic parameters, day respiration (optionally calculated from leaf temperature), and mesophyll conductance. 
 #' @param VPD Vapour pressure deficit (kPa) (not needed when RH provided)
 #' @param Ca Atmospheric CO2 concentration (ppm)
 #' @param PPFD Photosynthetic photon flux density ('PAR') (mu mol m-2 s-1)
 #' @param Tleaf Leaf temperature (degrees C)
 #' @param Patm Atmospheric pressure (kPa) (but see warning below!)
 #' @param RH Relative humidity (in \%) (not needed when VPD provided)
-#' @param gsmodel One of BBOpti (Medlyn et al. 2011), BBLeuning (Leuning 1995), or BallBerry (Ball et al. 1987)
+#' @param gsmodel One of BBOpti (Medlyn et al. 2011), BBLeuning (Leuning 1995), BallBerry (Ball et al. 1987), or BBdefine (for full control; see Details).
 #' @param g0,g1 Parameters of Ball-Berry type stomatal conductance models.
 #' @param gk Optional, exponent of VPD in gs model (Duursma et al. 2013)
 #' @param vpdmin Below vpdmin, VPD=vpdmin, to avoid very high gs.
 #' @param D0 Parameter for the BBLeuning stomatal conductance model.
 #' @param GS Optionally, stomatal conductance (to H2O). If provided, \code{Photosyn} calculates Ci and photosynthesis. See Details.
+#' @param BBmult Optional, only used when \code{gsmodel = "BBdefine"}, see Details.
 #' @param alpha Quantum yield of electron transport (mol mol-1)
 #' @param theta Shape of light response curve.
 #' @param Jmax Maximum rate of electron transport at 25 degrees C (mu mol m-2 s-1)
 #' @param Vcmax Maximum carboxylation rate at 25 degrees C (mu mol m-2 s-1)
 #' @param gmeso Mesophyll conductance (mol m-2 s-1). If not NULL (the default), Vcmax and Jmax are chloroplastic rates.
 #' @param TPU Triose-phosphate utilization rate (mu mol m-2 s-1); optional.
-#' @param Rd Dark respiration rate (mu mol m-2 s-1), optional (if not provided, calculated from Tleaf, Rd0, Q10 and TrefR)
-#' @param Rd0 Dark respiration rata at reference temperature (\code{TrefR})
+#' @param alphag Fraction of glycolate not returned to the chloroplast; parameter in TPU-limited photosynthesis (optional, only to be used when TPU is provided) (0 - 1)
+#' @param Rd Day respiration rate (mu mol m-2 s-1), optional (if not provided, calculated from Tleaf, Rd0, Q10 and TrefR)
+#' @param Rd0 Day respiration rate at reference temperature (\code{TrefR})
 #' @param Q10 Temperature sensitivity of Rd.
 #' @param TrefR Reference temperature for Rd (Celcius).
 #' @param Rdayfrac Ratio of Rd in the light vs. in the dark.
@@ -34,13 +36,20 @@
 #' @seealso \code{\link{FARAO}}, \code{\link{fitaci}}, \code{\link{AciC4}}
 #' @details The coupled photosynthesis - stomatal conductance model finds the intersection between the supply of CO2 by diffusion, and the demand for CO2 by photosynthesis. See Farquhar and Sharkey (1982) for basic description of this type of model, Duursma (2015) for more details on the implementation in the \code{plantecophys} package, and Duursma et al. (2014) for an example application (that uses this implementation).
 #' 
-#' The model of Farquhar et al. (1980) is used to estimate the dependence of leaf net photosynthesis rate (ALEAF) on intercellular CO2 concentration (Ci).   The temperature response of photosynthetic parameters, including Vcmax, Jmax, Gammastar, and Km follow Medlyn et al. (2002). 
-#'  
-#'  At the moment, two stomatal conductance models are implemented, both are Ball-Berry type models. The 'BBOpti' model is a slightly more general form of the model of Medlyn et al. 2011 (see Duursma et al. 2013). It is given by (in notation of the parameters and output variables of \code{Photosyn}),
+#' \strong{Photosynthesis model and temperature response} - 
+#' The model of Farquhar et al. (1980) is used to estimate the dependence of leaf net photosynthesis rate (ALEAF) on intercellular CO2 concentration (Ci), accounting for all three limitations (electron transport, carboxylation, and TPU limitation). The equations for the temperature response of photosynthetic parameters, including Vcmax, Jmax, Gammastar, and Km follow Medlyn et al. (2002). However, \strong{note that the default temperature response parameter values are not taken from Medlyn, and likely will have to be adjusted for your situation}.
+#' 
+#' #'  By default, the \code{Photosyn} function returns the hyperbolic minimum of Vcmax and Jmax-limited photosynthetic rates, as well as the hyperbolic minimum of Jmax-limited and TPU-limited rates. This approach avoids the discontinuity at the transition between the two rates (thus allowing use of \code{Photosyn} and \code{fitaci} in optimization or fitting routines). The individual rates (Ac, Aj and Ap) are also returned as output should they be needed. Note that those rates are output as gross photosynthetic rates (leaf respiration has to be subtracted to give net leaf photosynthesis).
+#' 
+#' \strong{Coupled leaf gas exchange}
+#'  When Ci is not provided, Ci is calculated from the intersection between the 'supply' and 'demand', where 'demand' is given by the Farquhar model of photosynthesis (A=f(Ci)), and supply by the stomatal conductance. The latter is, by default, estimated using the stomatal conductance model of Medlyn et al. (2011), but two other models are provided as well (Ball-Berry and Leuning, see \code{gsmodel} argument). Otherwise, stomatal conductance may be directly provided via the \code{GS} argument. 
+#' 
+#'  \strong{Stomatal conductance models} -  
+#'  At the moment, three stomatal conductance models are implemented. The 'BBOpti' model is a slightly more general form of the model of Medlyn et al. 2011 (see Duursma et al. 2013). It is given by (in notation of the parameters and output variables of \code{Photosyn}),
 #'  
 #'  \deqn{GS = g0 + 1.6*(1 + g1/D^(1-gk))*ALEAF/CA}
 #'  
-#'  where gk = 0.5 if stomata behave optimally (cf. Medlyn et al. 2011).
+#'  where \code{gk = 0.5} if stomata behave optimally (cf. Medlyn et al. 2011).
 #'  
 #'  The 'BBLeuning' model is that of Leuning (1995). It is given by,
 #'  
@@ -52,29 +61,42 @@
 #'  
 #'  \deqn{GS = g0 + g1*RH*ALEAF/Ca}
 #'  
-#'  Where RH is relative humidity.
+#'  Where RH is relative humidity. Again, the g1 value is not comparable to that used in the previous two models.
 #'  
-#'  For the full numerical solution to the Cowan-Farquhar optimization, use the \code{\link{FARAO}} function (which was used in Medlyn et al. 2011 for comparison to the approximation there presented).
+#'  Finally, \code{Photosyn} provides a very flexible Ball-Berry model, where the multiplier has to be specified by the user, the model is:
 #'  
-#'  If the mesophyll conductance \code{gmeso} is provided, it is assumed that Vcmax and Jmax are the chloroplastic rates, and leaf photosynthesis is calculated following Ethier and Livingston (2004).
+#'  \deqn{GS = g0 + BBmult*ALEAF}
 #'  
-#'  If Ci is provided as an input, this function calculates an A-Ci curve. Otherwise, Ci is calculated from the intersection between the 'supply' and 'demand', where 'demand' is given by the Farquhar model of photosynthesis (A=f(Ci)), and supply by the stomatal conductance. The latter is, by default, estimated using the stomatal conductance model of Medlyn et al. (2011), but two other models are provided as well (Ball-Berry and Leuning, see \code{gsmodel} argument). Otherwise, stomatal conductance may be directly provided via the \code{GS} argument. 
+#'  This interface can be used to quickly simulate what happens if stomata do not respond to humidity at all (in which case BBmult=g1/Ca, or ca. 5/400), or to use the Tuzet model of stomatal conductance inside another model that provides the leaf water potential function.
 #'  
-#' Note that the function \code{Aci} is provided as a shorthand for \code{Photosyn(Ci=x)}.
+#'  For the full numerical solution to the Cowan-Farquhar optimization, use the \code{\link{FARAO}} function (which was used in Medlyn et al. 2011 for comparison to the approximation there presented). See Duursma (2015) for more details.
 #'  
-#'  By default, the \code{Photosyn} function returns the hyperbolic minimum of Vcmax and Jmax-limited photosynthetic rates. This is to avoid the discontinuity at the transition between the two rates. Both Ac and Aj are also returned should they be needed. Note that those rates are output as gross photosynthetic rates (leaf respiration has to be subtracted to give net leaf photosynthesis).
-#' @section Atmospheric pressure:
+#'  \strong{Mesophyll conductance} - 
+#'  
+#'  If the mesophyll conductance \code{gmeso} is provided as an input, it is assumed that Vcmax and Jmax are the chloroplastic rates, and leaf photosynthesis is calculated following the equations from Ethier and Livingston (2004). When very low mesophyll conductance rates are input, the model may return poor solutions (and sometimes they may not exist).
+#'  
+#'  \strong{Simulating A-Ci curves}
+#'  
+#'  If Ci is provided as an input, this function calculates an A-Ci curve. For example, you may do \code{Photosyn(Ci=300)}, for which the function \code{Aci} is included as a shortcut (\code{Aci(300)}).
+#'  
+#' \strong{Atmospheric pressure} - 
 #' 
 #' A correction for atmospheric pressure (Patm) is implemented in \code{\link{fitaci}}, but \strong{not in Photosyn}. In \code{fitaci}, the necessary corrections are applied so that estimated Vcmax and Jmax are expressed at standard pressure (Patm=100kPa). In Photosyn, however, the corrections are much more complicated and tend to be very small, because effects of Patm on partial pressures are largely offset by increases in diffusivity (Terashima et al. 1995, Gale 1973). 
 #' 
 #' Note that Patm is an argument to the Photosyn function, but it only affects calculations of Km and GammaStar (as used by fitaci), and transpiration rate. Setting only Patm \strong{does not correct for atmospheric pressure effects on photosynthesis rates}.
 #' 
+#' The simulation of limitation of the photosynthetic rate to triose-phosphate utilization follows details in Ellsworth et al. (2015), their Eq. 7. Note that the parameter \code{alphag} is set to zero by default.
+#' 
+#' 
+#' 
 #' @references 
 #' Duursma, R.A., Payton, P., Bange, M.P., Broughton, K.J., Smith, R.A., Medlyn, B.E., Tissue, D. T., 2013,  Near-optimal response of instantaneous transpiration efficiency to vapour pressure deficit, temperature and [CO2] in cotton (Gossypium hirsutum L.). Agricultural and Forest Meteorology 168 : 168 - 176.
 #' 
 #' Duursma, R.A., Barton, C.V.M., Lin, Y.-S., Medlyn, B.E., Eamus, D., Tissue, D.T., Ellsworth, D.S., McMurtrie, R.E., 2014. The peaked response of transpiration rate to vapour pressure deficit in field conditions can be explained by the temperature optimum of photosynthesis. Agricultural and Forest Meteorology 189 - 190, 2-10. doi:10.1016/j.agrformet.2013.12.007
 #'
 #' Duursma, R.A., 2015. Plantecophys - An R Package for Analysing and Modelling Leaf Gas Exchange Data. PLoS ONE 10, e0143346. doi:10.1371/journal.pone.0143346
+#' 
+#' Ellsworth, D.S., Crous, K.Y., Lambers, H., Cooke, J., 2015. Phosphorus recycling in photorespiration maintains high photosynthetic capacity in woody species. Plant Cell Environ 38, 1142-1156. doi:10.1111/pce.12468
 #'
 #'Ethier, G. and N. Livingston. 2004. On the need to incorporate sensitivity to CO2 transfer conductance into the Farquhar von Caemmerer Berry leaf photosynthesis model. Plant, Cell & Environment. 27:137-153.
 #'
@@ -146,20 +168,22 @@ Photosyn <- function(VPD=1.5,
                      Patm=100,
                      RH=NULL,
                      
-                     gsmodel=c("BBOpti","BBLeuning","BallBerry"),
+                     gsmodel=c("BBOpti","BBLeuning","BallBerry","BBdefine"),
                      g1=4,
                      g0=0, 
                      gk=0.5,
                      vpdmin=0.5,
                      D0=5,
                      GS=NULL,
-                      
+                     BBmult=NULL,
+                     
                      alpha=0.24, 
                      theta=0.85, 
                      Jmax=100, 
                      Vcmax=50, 
                      gmeso=NULL,
                      TPU=1000,
+                     alphag=0,
                      
                      Rd0 = 0.92,
                      Q10 = 1.92,
@@ -186,6 +210,9 @@ Photosyn <- function(VPD=1.5,
   
   whichA <- match.arg(whichA)
   gsmodel <- match.arg(gsmodel)
+  if(gsmodel == "BBdefine" && is.null(BBmult)){
+    stop("When defining your own BB multiplier, set BBmult.")
+  }
   inputCi <- !is.null(Ci)
   inputGS <- !is.null(GS)
   
@@ -239,45 +266,53 @@ Photosyn <- function(VPD=1.5,
     GSDIVA <- g1 / Ca / (1 + VPD/D0)
     GSDIVA <- GSDIVA / GCtoGW   # convert to conductance to CO2
   }
+  # Original Ball&Berry 1987 model.
   if(gsmodel == "BallBerry"){
     if(is.null(RH))RH <- VPDtoRH(VPD, Tleaf)
     RH <- RH / 100
     GSDIVA <- g1 * RH / Ca
     GSDIVA <- GSDIVA / GCtoGW   # convert to conductance to CO2
-  }  
-  
+  } 
+  # Multiplier is user-defined.
+  if(gsmodel == "BBdefine"){
+    GSDIVA <- BBmult / GCtoGW
+  }
   
   if(inputGS){
+    
     GC <- GS / GCtoGW
     
-    # Solution when Rubisco activity is limiting
-    A <- 1./GC
-    B <- (Rd - Vcmax)/GC - Ca - Km
-    C <- Vcmax * (Ca - GammaStar) - Rd * (Ca + Km)
-    Ac <- (- B - sqrt(B*B - 4*A*C)) / (2*A)
-    
-    # Photosynthesis when electron transport is limiting
-    B <- (Rd - VJ)/GC - Ca - 2*GammaStar
-    C <- VJ * (Ca - GammaStar) - Rd * (Ca + 2*GammaStar)
-    Aj <- (- B - sqrt(B*B - 4*A*C)) / (2*A)
-    
-    # NOTE: this solution gives net photosynthesis (see old Maestra code).
-    Ac <- Ac + Rd
-    Aj <- Aj + Rd
+    if(GS > 0){
+      
+      # Solution when Rubisco activity is limiting
+      A <- 1./GC
+      B <- (Rd - Vcmax)/GC - Ca - Km
+      C <- Vcmax * (Ca - GammaStar) - Rd * (Ca + Km)
+      Ac <- QUADM(A,B,C)
+      
+      # Photosynthesis when electron transport is limiting
+      B <- (Rd - VJ)/GC - Ca - 2*GammaStar
+      C <- VJ * (Ca - GammaStar) - Rd * (Ca + 2*GammaStar)
+      Aj <- QUADM(A,B,C)
+      
+      # NOTE: the solution above gives net photosynthesis, add Rd
+      # to get gross rates (to be consistent with other solutions).
+      Ac <- Ac + Rd
+      Aj <- Aj + Rd
+    } else {
+      Ac <- Aj <- 0
+    }
     
   } else {
     
     # If CI not provided, calculate from intersection between supply and demand
     if(!inputCi){
     
-      #--- non-vectorized workhorse
-      #! This can be vectorized, if we exclude Zero PPFD first,
-      #! move whichA to somewhere else (at the end), and don't take minimum
-        #! of Ac, Aj (calc both, return pmin at the end).
+        #--- non-vectorized workhorse
         getCI <- function(VJ,GSDIVA,PPFD,VPD,Ca,Tleaf,vpdmin,g0,Rd,
                               Vcmax,Jmax,Km,GammaStar){
           
-          if(PPFD == 0){
+          if(identical(PPFD, 0) | identical(VJ, 0)){
             vec <- c(Ca,Ca)
             return(vec)
           }
@@ -290,7 +325,7 @@ Photosyn <- function(VPD=1.5,
             (Km - Ca)- GSDIVA * (Vcmax*GammaStar + Km*Rd)
           C <- -(1. - Ca*GSDIVA) * (Vcmax*GammaStar + Km*Rd) - g0*Km*Ca
           
-          CIC <- (- B + sqrt(B*B - 4*A*C)) / (2*A)
+          CIC <- QUADP(A,B,C)
           
           # Solution when electron transport rate is limiting
           A <- g0 + GSDIVA * (VJ - Rd)
@@ -299,11 +334,7 @@ Photosyn <- function(VPD=1.5,
           C <- -(1 - Ca*GSDIVA) * GammaStar * (VJ + 2*Rd) - 
             g0*2*GammaStar*Ca
           
-          if(A == 0)
-            CIJ <- -C/B
-          else
-            CIJ <- (- B + sqrt(B*B - 4*A*C)) / (2*A)
-          
+          CIJ <- QUADP(A,B,C)
           return(c(CIJ,CIC))
         }
       
@@ -336,9 +367,9 @@ Photosyn <- function(VPD=1.5,
         CIJ <- Ci
         
         if(length(GammaStar) > 1){
-          CIJ[CIJ < GammaStar] <- GammaStar[CIJ < GammaStar]
+          CIJ[CIJ <= GammaStar] <- GammaStar[CIJ < GammaStar]
         } else {
-          CIJ[CIJ < GammaStar] <- GammaStar
+          CIJ[CIJ <= GammaStar] <- GammaStar
         }
         
         CIC <- Ci
@@ -386,18 +417,20 @@ Photosyn <- function(VPD=1.5,
   }
 
     # Limitation by triose-phosphate utilization
-    Ap <- 3*TPU   
-    
+    if(!is.null(Ci)){
+      Ap <- 3 * TPU * (Ci - GammaStar)/(Ci - (1 + 3*alphag)*GammaStar)   
+      Ap[Ci < 400] <- 1000  # avoid nonsense
+    } else {
+      Ap <- 1000  # This is when inputGS = TRUE; 
+    }  
+  
     # Hyperbolic minimum.
-    hmshape <- 0.9999
-    Am <- (Ac+Aj - sqrt((Ac+Aj)^2-4*hmshape*Ac*Aj))/(2*hmshape)
-    
+    Am <- -mapply(QUADP, A = 1 - 1E-04, B = Ac+Aj, C = Ac*Aj)
+
     # Another hyperbolic minimum with the transition to TPU
-    #browser()
     tpulim <- any(Ap < Am)
     if(!is.na(tpulim) && tpulim){
-      hmshape <- 1 - 1E-07
-      Am <- (Am+Ap - sqrt((Am+Ap)^2-4*hmshape*Am*Ap))/(2*hmshape)
+      Am <- -mapply(QUADP, A = 1 - 1E-07, B = Am+Ap, C = Am*Ap)
     }
     
     # Net photosynthesis
@@ -426,6 +459,17 @@ Photosyn <- function(VPD=1.5,
     # Calculate Ci if GS was provided as input.
     if(inputGS){
       Ci <- Ca - Am/GC
+      
+      # For zero GC:
+      Ci[!is.finite(Ci)] <- Ca
+      # Stomata fully shut; Ci is not really Ca but that's how we like to think about it.
+    }
+    
+    # Chloroplastic CO2 concentration
+    if(!is.null(gmeso)){
+      Cc <- Ci - Am/gmeso
+    } else {
+      Cc <- Ci
     }
     
     # Transpiration rate assuming perfect coupling.
@@ -443,6 +487,7 @@ Photosyn <- function(VPD=1.5,
                       VPD=VPD,
                       Tleaf=Tleaf,
                       Ca=Ca,
+                      Cc=Cc,
                       PPFD=PPFD,
                       Patm=Patm)
   
@@ -462,6 +507,7 @@ Jfun <- function(PPFD, alpha, Jmax, theta){
      sqrt((alpha*PPFD + Jmax)^2 - 4*alpha*theta*PPFD*Jmax))/(2*theta)
 }
 
+# Given PPFD and J, what is Jmax? (inverse non-rectangular hyperbola)
 inverseJfun <- function(PPFD, alpha, J, theta){
   J*(J*theta - alpha*PPFD)/(J - alpha*PPFD)
 }
